<?php
/*
 * SSD Analyzer module
 * @version 0.8
 */
global $session;

// 0. Запуск Self-test по запросу
if (!empty($_REQUEST['run_selftest'])) {
    $testDev = $_REQUEST['run_selftest'];
    $msg = shell_exec("sudo smartctl -t long " . escapeshellarg("/dev/{$testDev}") . " 2>&1");
    $out['SELFTEST_TRIGGERED'][] = [
        'DEV'     => $testDev,
        'MESSAGE' => trim($msg)
    ];
}

// 1. Проверка ОС
if (stripos(PHP_OS, 'Linux') === false
    && stripos(PHP_OS, 'Darwin') === false
    && stripos(PHP_OS, 'FreeBSD') === false) {
    $out['ERROR'] = 'Поддерживаются только Unix-системы';
    return;
}

// 2. Обнаружение утилит
function detectUtilities() {
    $bins = [
        'lsblk'    => 'util-linux',
        'hdparm'   => 'hdparm',
        'smartctl' => 'smartmontools',
        'nvme'     => 'nvme-cli'
    ];
    $res = [];
    foreach ($bins as $bin => $pkg) {
        $path = trim(shell_exec(escapeshellcmd("command -v {$bin}")));
        $ok   = ($path !== '');
        $res[$bin] = [
            'installed' => $ok,
            'path'      => $ok ? $path : '',
            'message'   => $ok ? '' : "sudo apt install {$pkg}"
        ];
    }
    return $res;
}

// 3. Парсеры
function flattenJson($data, $prefix = '') {
    $out = [];
    foreach ($data as $k => $v) {
        $key = $prefix ? "{$prefix}.{$k}" : $k;
        if (is_array($v)) {
            if (array_values($v) === $v) {
                $out[] = ['KEY' => $key, 'VALUE' => json_encode($v)];
            } else {
                $out = array_merge($out, flattenJson($v, $key));
            }
        } else {
            $out[] = ['KEY' => $key, 'VALUE' => (string)$v];
        }
    }
    return $out;
}

function getLsblkFullInfo($dev) {
    $out  = shell_exec("lsblk -O -J " . escapeshellarg("/dev/{$dev}"));
    $json = json_decode($out, true);
    if (empty($json['blockdevices'][0])) return [];
    return flattenJson($json['blockdevices'][0]);
}

function getHdparmRaw($dev) {
    $out   = shell_exec("sudo hdparm -I " . escapeshellarg("/dev/{$dev}") . " 2>&1");
    $lines = preg_split('/\r?\n/', $out);
    $res   = [];
    $key   = null;
    $value = '';
    foreach ($lines as $line) {
        if (preg_match('/^(\S.*?):\s*(.*)$/', $line, $m)) {
            if ($key !== null) {
                $res[] = ['KEY' => $key, 'VALUE' => trim($value)];
            }
            $key   = trim($m[1]);
            $value = trim($m[2]);
        } elseif ($key !== null && trim($line) !== '') {
            $value .= ' ' . trim($line);
        }
    }
    if ($key !== null) {
        $res[] = ['KEY' => $key, 'VALUE' => trim($value)];
    }
    return $res;
}

function getSmartctlJson($dev) {
    $out = shell_exec("sudo smartctl -H -A -x -j " . escapeshellarg("/dev/{$dev}") . " 2>&1");
    return json_decode($out, true) ?: [];
}

function getSmartctlFullRaw($dev) {
    return shell_exec("sudo smartctl -a " . escapeshellarg("/dev/{$dev}") . " 2>&1");
}

// Новый: JSON-парсер Self-test
function getSmartctlSelfTestJson($dev) {
    $out = shell_exec("sudo smartctl -l selftest -j " . escapeshellarg("/dev/{$dev}") . " 2>&1");
    return json_decode($out, true) ?: [];
}

function getSmartctlSelfTestRaw($dev) {
    return shell_exec("sudo smartctl -l selftest " . escapeshellarg("/dev/{$dev}") . " 2>&1");
}

function getNvmeCtrl($dev) {
    $ctrl = preg_replace('/n\d+$/','',$dev);
    $j    = shell_exec("sudo nvme id-ctrl -o json " . escapeshellarg("/dev/{$ctrl}") . " 2>&1");
    return flattenJson(json_decode($j, true) ?: []);
}

function getNvmeNs($dev) {
    $j = shell_exec("sudo nvme id-ns -o json " . escapeshellarg("/dev/{$dev}") . " 2>&1");
    return flattenJson(json_decode($j, true) ?: []);
}

function getNvmeSmart($dev) {
    $ctrl = preg_replace('/n\d+$/','',$dev);
    $j    = shell_exec("sudo nvme smart-log -o json " . escapeshellarg("/dev/{$ctrl}") . " 2>&1");
    return flattenJson(json_decode($j, true) ?: []);
}

// 4. Оценка здоровья базовая (без self-test)
function evaluateHealth($json) {
    $parts = [];
    $table = $json['ata_smart_attributes']['table'] ?? [];
    $hours = 0; $realloc = 0; $wear = null;
    foreach ($table as $a) {
        if ($a['name']==='Power_On_Hours')        $hours   = (int)$a['raw']['value'];
        if ($a['name']==='Reallocated_Sector_Ct') $realloc = (int)$a['raw']['value'];
    }
    if ($hours)   $parts[] = "отработал {$hours} ч";
    if ($realloc) $parts[] = "перераспр. {$realloc} секторов";
    if (isset($json['nvme_smart_health_information_log']['percentage_used'])) {
        $wear = (int)$json['nvme_smart_health_information_log']['percentage_used'];
        $parts[] = "износ {$wear}%";
    }
    $state = 'состояние хорошее';
    if ($wear !== null) {
        if ($wear > 80)      $state = 'состояние критическое';
        elseif ($wear > 50)  $state = 'состояние требует внимания';
    }
    $parts[] = $state;
    $level = ($wear !== null && $wear > 80) ? 'danger'
        : ($wear !== null && $wear > 50) ? 'warning'
            : 'success';
    $exp = [];
    if (isset($json['smart_status']['passed'])) {
        $passed = $json['smart_status']['passed'];
        $exp[] = "smartctl: общий статус S.M.A.R.T.=" . ($passed ? 'PASSED — всё отлично' : 'FAILED — серьёзная проблема');
    }
    foreach ($table as $a) if ($a['name']==='Reallocated_Sector_Ct') {
        $v = (int)$a['raw']['value'];
        $exp[] = "smartctl: Reallocated_Sector_Ct={$v} — " . ($v === 0 ? 'нигде не переназначалось' : 'следить за ростом');
    }
    foreach ($table as $a) if ($a['name']==='Temperature_Celsius') {
        $v = (int)$a['raw']['value'];
        $n = $v <= 50 ? 'нормальная температура' : ($v >= 60 ? 'перегрев' : '');
        $exp[] = "smartctl: Temperature_Celsius={$v}°C — {$n}";
    }
    $nvh = $json['nvme_smart_health_information_log'] ?? [];
    if (isset($nvh['percentage_used'])) {
        $v = (int)$nvh['percentage_used'];
        $n = $v <= 30 ? 'ресурс не тронут' : ($v <= 70 ? 'готовьте бэкапы' : 'риски');
        $exp[] = "nvme: percentage_used={$v}% — {$n}";
    }
    if (isset($nvh['critical_warning'])) {
        $v = (int)$nvh['critical_warning'];
        $exp[] = "nvme: critical_warning={$v} — " . ($v === 0 ? 'нет аварийных предупреждений' : 'срочно выяснять причину');
    }
    if (isset($nvh['data_units_written'])) {
        $exp[] = "nvme: data_units_written={$nvh['data_units_written']} — просто мониторьте рост";
    }
    if (isset($nvh['host_read_commands']) || isset($nvh['host_write_commands'])) {
        $r = $nvh['host_read_commands']  ?? 0;
        $w = $nvh['host_write_commands'] ?? 0;
        $exp[] = "nvme: host_read_commands={$r}, host_write_commands={$w} — оценка «пробега»";
    }

    return [
        'text'        => implode(', ', $parts),
        'level'       => $level,
        'explanation' => $exp
    ];
}

// 5. Собираем отчёт
try {
    $utils = detectUtilities();

    // 5.1. Статус утилит
    $out['UTILITIES'] = []; $errs = [];
    foreach ($utils as $n => $u) {
        $out['UTILITIES'][] = [
            'NAME'      => $n,
            'INSTALLED' => $u['installed'],
            'PATH'      => $u['path'],
            'MESSAGE'   => $u['message']
        ];
        if (!$u['installed']) {
            $errs[] = ['MESSAGE' => "Утилита {$n} не установлена. Установите её: {$u['message']}"];
        }
    }
    $out['ERRORS'] = $errs;

    // 5.2. Список дисков
    $out['DEVICES'] = [];
    if ($utils['lsblk']['installed']) {
        $all = json_decode(shell_exec("lsblk -O -J"), true);
        foreach (($all['blockdevices'] ?? []) as $d) {
            if (($d['type'] ?? '') !== 'disk') continue;
            $dev = $d['name'];

            // SMART JSON и базовое здоровье
            $sj = $utils['smartctl']['installed'] ? getSmartctlJson($dev) : [];
            $h  = evaluateHealth($sj);

            // Self-test JSON
            $stj    = $utils['smartctl']['installed'] ? getSmartctlSelfTestJson($dev) : [];
            // ← здесь берём реальные данные из JSON
            $tests  = $stj['ata_smart_self_test_log']['standard']['table'] ?? [];

            $sttext = '';
            $stlevel = null;
            if (count($tests)) {
                $last = end($tests);
                $passed = $last['status']['passed'] ?? false;
                $statusStr = $last['status']['string'] ?? '';
                if ($passed) {
                    $sttext  = "self test=без ошибок";
                    $stlevel = 'success';
                } else {
                    $sttext  = "self test=" . mb_strtolower($statusStr);
                    $stlevel = 'danger';
                }
                // Добавляем эту строку к пояснениям
                $h['explanation'][] = "smartctl: {$sttext}";
                // Пересчитываем общий статус-плашку
                if ($stlevel === 'danger') {
                    $statusText  = 'Self-test FAILED';
                    $statusLevel = 'danger';
                } else {
                    $statusText  = ucfirst($h['text']) . ', self-test OK';
                    $statusLevel = $h['level'];
                }
            } else {
                // без данных Self-test
                $statusText  = ucfirst($h['text']);
                $statusLevel = $h['level'];
            }

            // Собираем всю информацию по диску
            $info = [
                'NAME'                  => $dev,
                'LSBLK'                 => getLsblkFullInfo($dev),
                'HDPARM'                => $utils['hdparm']['installed']   ? getHdparmRaw($dev)          : [],
                'SMARTCTL_FULL_RAW'     => $utils['smartctl']['installed']  ? getSmartctlFullRaw($dev)     : '',
                'SMARTCTL_SELFTEST_RAW' => $utils['smartctl']['installed']  ? getSmartctlSelfTestRaw($dev) : '',
                'SMARTCTL_ATTRS'        => $sj['ata_smart_attributes']['table']
                    ? array_map(function($a){
                        return [
                            'ID'             => $a['id'],
                            'ATTRIBUTE_NAME' => $a['name'],
                            'VALUE'          => $a['value'],
                            'WORST'          => $a['worst'],
                            'THRESH'         => $a['thresh'],
                            'RAW_VALUE'      => is_scalar($a['raw']['value'])
                                ? $a['raw']['value']
                                : json_encode($a['raw']['value'])
                        ];
                    }, $sj['ata_smart_attributes']['table'])
                    : [],
                'NVME_CTRL'             => $utils['nvme']['installed']     ? getNvmeCtrl($dev)            : [],
                'NVME_NS'               => $utils['nvme']['installed']     ? getNvmeNs($dev)              : [],
                'NVME_SMART'            => $utils['nvme']['installed']     ? getNvmeSmart($dev)           : [],
                'STATUS_TEXT'           => $statusText,
                'STATUS_LEVEL'          => $statusLevel,
                'HEALTH_EXPLANATION'    => array_map(function($t){ return ['ITEM'=>$t]; }, $h['explanation'])
            ];
            $out['DEVICES'][] = $info;
        }
    }
} catch (Exception $e) {
    $out['ERROR'] = $e->getMessage();
}
